#!/usr/bin/env python
# coding: utf-8

from .main import main as main1
from .saas import main as main2

def main():
    def parse_args():
        import argparse
        parser = argparse.ArgumentParser(description="Python use - AIPython")
        parser.add_argument("-c", '--config', type=str, default=None, help="Toml config file")
        parser.add_argument('--python', default=False, action='store_true', help="Python mode")
        return parser.parse_args()
    args = parse_args()
    if args.python:
        main1(args)
    else:
        main2(args)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pathlib import Path
import importlib.resources as resources

from dynaconf import Dynaconf
from rich.console import Console
from prompt_toolkit import PromptSession
from prompt_toolkit.history import FileHistory
from prompt_toolkit.styles import Style

from .aipy import Agent
from .aipy.i18n import T

__PACKAGE_NAME__ = "aipyapp"

class InteractiveConsole():
    def __init__(self, ai, console, settings):
        self.ai = ai
        self.history = FileHistory(str(Path.cwd() / settings.history))
        self.session = PromptSession(history=self.history)
        self.console = console
        self.style_main = Style.from_dict({"prompt": "green"})
        self.style_ai = Style.from_dict({"prompt": "cyan"})

    def input_with_possible_multiline(self, prompt_text, is_ai=False):
        prompt_style = self.style_ai if is_ai else self.style_main

        first_line = self.session.prompt([("class:prompt", prompt_text)], style=prompt_style)
        if not first_line.endswith("\\"):
            return first_line
        # Multi-line input
        lines = [first_line.rstrip("\\")]
        while True:
            next_line = self.session.prompt([("class:prompt", "... ")], style=prompt_style)
            if next_line.endswith("\\"):
                lines.append(next_line.rstrip("\\"))
            else:
                lines.append(next_line)
                break
        return "\n".join(lines)

    def run_ai_task(self, task):
        try:
            self.ai(task)
        except Exception as e:
            self.console.print(f"[bold red]Error: {e}")

    def run_ai_mode(self, initial_text):
        ai = self.ai
        self.console.print(f"{T('ai_mode_enter')}", style="cyan")
        self.run_ai_task(initial_text)
        while True:
            try:
                user_input = self.input_with_possible_multiline(">>> ", is_ai=True).strip()
            except (EOFError, KeyboardInterrupt):
                break
            if not user_input:
                continue

            if user_input.startswith("/"):
                if user_input.startswith("/done"):
                    break
                elif user_input.startswith("/use "):
                    llm = user_input[5:].strip()
                    if llm: ai.use(llm)
                else:
                    self.console.print(f"{T('ai_mode_unknown_command')}", style="cyan")
            else:
                self.run_ai_task(user_input)
        try:
            ai.publish(verbose=False)
        except Exception as e:
            pass
        try:
            ai.done()
        except Exception as e:
            self.console.print_exception()
            pass
        self.console.print(f"{T('ai_mode_exit')}", style="cyan")

    def run(self):
        names = self.ai.llm.names
        self.console.print(f"{T('banner1')}", style="green")
        self.console.print(f"[cyan]{T('default')}: [green]{names['default']}，[cyan]{T('available')}: [yellow]{' '.join(names['available'])}")
        while True:
            try:
                user_input = self.input_with_possible_multiline(">> ").strip()
                if len(user_input) < 2:
                    continue
                if user_input.startswith("/use "):
                    llm = user_input[5:].strip()
                    if llm: self.ai.use(llm)
                else:
                    self.run_ai_mode(user_input)
            except (EOFError, KeyboardInterrupt):
                break

def main(args):
    console = Console(record=True)
    console.print("[bold cyan]🚀 Python use - AIPython ([red]Task mode[/red])")

    path = args.config if args.config else 'aipython.toml'
    default_config_path = resources.files(__PACKAGE_NAME__) / "default.toml"
    settings = Dynaconf(settings_files=[str(default_config_path), path], envvar_prefix="AIPY", merge_enabled=True)
    try:
        ai = Agent(settings, console=console)
    except Exception as e:
        console.print_exception(e)
        console.print(f"[bold red]Error: {e}")
        return
    
    if not ai.llm:
        console.print(f"[bold red]{T('no_available_llm')}")
        return
  
    InteractiveConsole(ai, console, settings).run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import subprocess
from functools import wraps

from .i18n import T

def restore_output(func):
    @wraps(func)
    def wrapper(self, *args, **kwargs):
        old_stdout, old_stderr = sys.stdout, sys.stderr
        sys.stdout, sys.stderr = sys.__stdout__, sys.__stderr__

        try:
            return func(self, *args, **kwargs)
        finally:
            sys.stdout, sys.stderr = old_stdout, old_stderr
    return wrapper

def confirm(console, prompt, default="n", auto=None):
    if auto in (True, False):
        console.print(f"✅ {T('auto_confirm')}")
        return auto
    while True:
        response = console.input(prompt).strip().lower()
        if not response:
            response = default
        if response in ["y", "n"]:
            break
    return response == "y"

def get_uv_from_venv():
    venv_path = os.getenv("VIRTUAL_ENV")
    if not venv_path:
        print("VIRTUAL_ENV is not set.")
        return None
    
    cfg_path = os.path.join(venv_path, "pyvenv.cfg")
    if not os.path.exists(cfg_path):
        print(f"pyvenv.cfg not found in {venv_path}")
        return None
    
    uv_value = None
    with open(cfg_path, "r", encoding="utf-8") as f:
        for line in f:
            line = line.strip()
            if line.startswith("uv ="):
                _, uv_value = map(str.strip, line.split("=", 1))
                break
    
    return uv_value

def uv_install_packages(console, packages):
    uvv = get_uv_from_venv()
    if not uvv:
        console.print("[red]Not in uv venv, can't install packages")
        return False
    
    console.print(f"[green]Install packages: {', '.join(packages)} with uv {uvv}")
    cp = subprocess.run(['uv', "add"] + packages)
    return cp.returncode == 0


#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import time
import json
import uuid
import requests
from enum import Enum
from pathlib import Path

from rich.live import Live
from rich.panel import Panel
from rich.align import Align
from rich.table import Table
from rich.syntax import Syntax
from rich.console import Console
from rich.markdown import Markdown

from . import i18n
from . import utils
from .i18n import T
from .llm import LLM
from .runner import Runner

class MsgType(Enum):
    CODE = "CODE"
    TEXT = "TEXT"

class Agent():
    MAX_TOKENS = 4096
    CERT_PATH = Path('/tmp/aipy_client.crt')

    def __init__(self, settings, console=None):
        self.settings = settings
        self.instruction = None
        self.llm = None
        self.runner = None
        self._console = console
        self.system_prompt = None
        self.max_tokens = None
        self._cwd = None
        self.task_id = None
        self._init()

    def _init(self):
        config = self.settings
        lang = config.get('lang')
        if lang:
            i18n.lang = lang
        self._console = self._console or Console(record=config.get('record', True))
        self.max_tokens = config.get('max_tokens', self.MAX_TOKENS)
        self.system_prompt = config.get('system_prompt')
        self.runner = Runner(self._console, config)
        self.llm = LLM(self._console,config['llm'], self.max_tokens)
        self.use = self.llm.use
        if config.workdir:
            workdir = Path.cwd() / config.workdir
            workdir.mkdir(parents=True, exist_ok=True)
            os.chdir(workdir)
            self._cwd = workdir
        else:
            self._cwd = Path.cwd()

        api = config.get('api')
        if api:
            lines = [self.system_prompt]
            for api_name, api_conf in api.items():
                lines.append(f"## {api_name} API")
                envs = api_conf.get('env', {})
                if envs:
                    lines.append(f"### {T('env_description')}")
                    for name, (value, desc) in envs.items():
                        value = value.strip()
                        if not value:
                            continue
                        var_name = name
                        lines.append(f"- {var_name}: {desc}")
                        self.runner.setenv(var_name, value, desc)
                desc = api_conf.get('desc')
                if desc: 
                    lines.append(f"### API {T('description')}\n{desc}")
            self.system_prompt = "\n".join(lines)

    def save(self, path):
        self._console.save_html(path, clear=False)
        
    def done(self):
        #self._console.save_svg('console.svg', clear=False)
        self._console.save_html('console.html', clear=True)
        task = {'instruction': self.instruction}
        task['llm'] = self.llm.history.json()
        task['runner'] = self.runner.history
        try:
            json.dump(task, open('task.json', 'w'), ensure_ascii=False, indent=4)
        except Exception as e:
            self._console.print_exception()
        self.llm.clear()
        self.runner.clear()
        self.task_id = None
        self.instruction = None
            
    def render_code(self, logs, language="python", max_lines=3, delay=0.1):
        console = self._console
        display_lines = []

        console.record = False
        with Live(console=console, refresh_per_second=10, vertical_overflow="crop") as live:
            for line in logs:
                display_lines.append(line)
                if len(display_lines) > max_lines:
                    display_lines.pop(0)

                code_block = "\n".join(display_lines)
                syntax = Syntax(code_block, language, theme="monokai", line_numbers=False, word_wrap=True)
                live.update(syntax)
                time.sleep(delay)
        console.record = True
        
    def parse_reply(self, text):
        lines = text.split('\n')
        code_block = []
        in_code_block = False
        in_run_block = False
        start = 0
        end = 0
        for i, line in enumerate(lines):
            if line.strip().startswith('```run'): 
                in_code_block = True
                in_run_block = True
                start = i
                continue
            elif line.strip().lower().startswith('```python'):
                in_code_block = True
                start = i
                continue
            elif line.strip().startswith('```') and in_run_block:
                end = i
                break
            if in_code_block and line.find('#RUN') >= 0:
                in_run_block = True

        if end > 0:
            code_block = lines[start+1:end]
            ret = {'type': MsgType.CODE, 'code': '\n'.join(code_block)}
        else:
            ret = {'type': MsgType.TEXT, 'code': None}
        return ret
        
    def process_code_reply(self, msg, llm=None):
        code_block = msg['code']
        self.box(f"\n⚡ {T('start_execute')}:", code_block, lang='python')
        result = self.runner(code_block)
        result = json.dumps(result, ensure_ascii=False)
        self._console.print(f"✅ {T('execute_result')}:\n", Markdown(f"```json\n{result}\n```"))
        self._console.print(f"\n📤 {T('start_feedback')}")
        feedback_response = self.llm(result, name=llm)
        return feedback_response

    def box(self, title, content, align=None, lang=None):
        if lang:
            content = Syntax(content, lang, line_numbers=True)
        if align:
            content = Align(content, align=align)
        self._console.print(Panel(content, title=title))

    def print_summary(self):
        history = self.llm.history
        """
        table = Table(title=T("Task Summary"), show_lines=True)

        table.add_column(T("Round"), justify="center", style="bold cyan", no_wrap=True)
        table.add_column(T("Time(s)"), justify="right")
        table.add_column(T("In Tokens"), justify="right")
        table.add_column(T("Out Tokens"), justify="right")
        table.add_column(T("Total Tokens"), justify="right", style="bold magenta")

        round = 1
        for row in history.get_usage():
            table.add_row(
                str(round),
                str(row["time"]),
                str(row["input_tokens"]),
                str(row["output_tokens"]),
                str(row["total_tokens"]),
            )
            round += 1
        self._console.print("\n")
        self._console.print(table)
        """
        summary = history.get_summary()
        if 'time' in summary:
            summary = "| {rounds} | {time:.3f}s | Tokens: {input_tokens}/{output_tokens}/{total_tokens}".format(**summary)
        else:
            summary = ''
        self._console.print(f"\n⏹ [cyan]{T('end_instruction')} {summary}")
    
    def __call__(self, instruction, llm=None):
        """
        执行自动处理循环，直到 LLM 不再返回代码消息
        """
        self.box(f"[yellow]{T('start_instruction')}", f'[red]{instruction}', align="center")
        system_prompt = None if self.llm.history else self.system_prompt
        if system_prompt:
            self.task_id = uuid.uuid4().hex
            self.instruction = instruction
            path = self._cwd / self.task_id
            path.mkdir(parents=True, exist_ok=False)
            os.chdir(path)
        response = self.llm(instruction, system_prompt=system_prompt, name=llm)
        while response:
            msg = self.parse_reply(response)
            if msg['type'] != MsgType.CODE:
                break
            response = self.process_code_reply(msg, llm)
        self.print_summary()
        os.write(1, b'\a\a\a')

    def chat(self, prompt):
        system_prompt = None if self.llm.history else self.system_prompt
        response, ok = self.llm(prompt, system_prompt=system_prompt)
        self._console.print(Markdown(response))

    def step(self):
        response = self.llm.get_last_message()
        if not response:
            self._console.print(f"❌ {T('no_context')}")
            return
        self.process_reply(response)

    def publish(self, title=None, author=None, verbose=True):
        url = self.settings.get('publish.url')
        cert = self.settings.get('publish.cert')
        if not (url and cert) or self.settings.get('publish.disable'):
            if verbose: self._console.print(f"[red]{T('publish_disabled')}")
            return False
        title = title or self.instruction
        author = author or os.getlogin()
        meta = {'author': author}
        files = {'content': self._console.export_html(clear=False)}
        data = {'title': title, 'metadata': json.dumps(meta)}

        if not (self.CERT_PATH.exists() and self.CERT_PATH.stat().st_size  > 0):
            self.CERT_PATH.write_text(cert)

        try:
            response = requests.post(url, files=files, data=data, cert=str(self.CERT_PATH), verify=True)
        except Exception as e:
            self._console.print_exception(e)
            return
        
        status_code = response.status_code
        if status_code in (200, 201):
            if verbose: self._console.print(f"[green]{T('upload_success')}:", response.json())
            return response.json()
        else:
            if verbose: self._console.print(f"[red]{T('upload_failed', status_code)}:", response.text)
            return False